﻿using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Kivii.Client.Invoices.Etax
{
    public partial class FrmMain : Form
    {
        private JsonServiceClient _client = null;

        public FrmMain()
        {
            InitializeComponent();
        }

        #region 界面控件默认值管理
        private void loadDefault(params Control[] ctls)
        {
            using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
            {
                foreach (var ctl in ctls)
                {
                    var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                    if (setting == null) continue;
                    ctl.Text = setting.ControlValue;
                }
            }
        }

        private void setDefault(params Control[] ctls)
        {
            using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
            {
                foreach (var ctl in ctls)
                {
                    var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                    if (setting == null)
                    {
                        setting = new UiSetting();
                        setting.FormName = this.Name;
                        setting.ControlName = ctl.Name;
                        setting.ControlValue = ctl.Text;
                        conn.Insert(setting);
                    }
                    else
                    {
                        setting.ControlValue = ctl.Text;
                        conn.Update(setting);
                    }
                }
            }
        }

        /// <summary>
        /// 输出消息
        /// </summary>
        /// <param name="message">消息内容</param>
        /// <param name="clearMessage">是否清除消息</param>
        private void outputMessage(string message, bool clearMessage = false)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(() => outputMessage(message, clearMessage)));
                return;
            }
            if (clearMessage) rtbMessage.Clear();
            if (rtbMessage.Lines.Count() > 500) rtbMessage.Clear();
            var time = DateTime.Now.ToString("HH:mm:ss");
            rtbMessage.AppendText($"[{time}]{message}{Environment.NewLine}");
            rtbMessage.ScrollToCaret();
        }

        #endregion

        private void FrmMain_Load(object sender, EventArgs e)
        {
            loadDefault(tbxServiceUrl, tbxUserName, tbxPassword, tbFilePath);
        }

        private void btnLogin_Click(object sender, EventArgs e)
        {
            if (_client != null)
            {
                MessageBox.Show("登陆前先退出原连接");
                return;
            }
            if (tbFilePath.Text.IsNullOrEmpty())
            {
                MessageBox.Show("请先设置读取路径！");
                return;
            }
            btnLogin.Enabled = false;
            _client = new JsonServiceClient(tbxServiceUrl.Text);
            var auth = new Authenticate { UserName = tbxUserName.Text, Password = tbxPassword.Text, provider = "Kivii" };
            var task = _client.PostAsync(auth);
            task.Success(resp =>
            {
                btnLogin.Enabled = resp.Kvid == Guid.Empty ? true : false;
                btnLogout.Enabled = !btnLogin.Enabled;
                tbxServiceUrl.Enabled = btnLogin.Enabled;
                tbxUserName.Enabled = btnLogin.Enabled;
                tbxPassword.Enabled = btnLogin.Enabled;
                btnSelectFileFolder.Enabled = btnLogin.Enabled;
                setDefault(tbxServiceUrl, tbxUserName, tbxPassword,tbFilePath);
                outputMessage($"登录成功：{resp.FullName}");
            }, true);
            task.Error(err =>
            {
                btnLogin.Enabled = true;
                btnLogout.Enabled = !btnLogin.Enabled;
                tbxServiceUrl.Enabled = btnLogin.Enabled;
                tbxUserName.Enabled = btnLogin.Enabled;
                tbxPassword.Enabled = btnLogin.Enabled;
                btnSelectFileFolder.Enabled = btnLogin.Enabled;
                _client.Dispose();
                _client = null;
                MessageBox.Show(err.Message, "Login Error");
            }, true);
        }

        private void btnLogout_Click(object sender, EventArgs e)
        {
            if (_client == null)
            {
                MessageBox.Show("未连接");
                return;
            }
            btnLogout.Enabled = false;
            var auth = new Authenticate { provider = "Logout" };
            var task = _client.PostAsync(auth);
            task.Success(resp =>
            {
                btnLogin.Enabled = resp != null;
                btnLogout.Enabled = !btnLogin.Enabled;
                tbxServiceUrl.Enabled = btnLogin.Enabled;
                tbxUserName.Enabled = btnLogin.Enabled;
                tbxPassword.Enabled = btnLogin.Enabled;
                btnSelectFileFolder.Enabled = btnLogin.Enabled;
                outputMessage($"退出成功");
                _client.Dispose();
                _client = null;
            });
            task.Error(err =>
            {
                btnLogout.Enabled = true;
                MessageBox.Show(err.Message, "Logout Error");
            });
        }

        private void btnSelectFileFolder_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            if (fbd.ShowDialog() != DialogResult.OK) return;
            tbFilePath.Text = fbd.SelectedPath;
            setDefault(tbFilePath);
        }
    }
}
